#ifndef CONFIG_H
#define CONFIG_H

#include <trace_buf.h>
#include <transport.h>

#ifndef PATH_MAX
#define PATH_MAX 512
#endif

#undef _EXTERN_
#ifndef __CONFIG_C__
#define _EXTERN_ extern
#else
#define _EXTERN_
#endif

#define SECONDS_PER_MINUTE 60
#define SECONDS_PER_HOUR 3600
#define SECONDS_PER_DAY 86400

// Configuration parameters
typedef struct {
   char sta[TRACE2_STA_LEN];      /* Site name (NULL-terminated) */
   char net[TRACE2_NET_LEN];      /* Network name (NULL-terminated) */
   char chan[TRACE2_CHAN_LEN];    /* Component/channel code (NULL-terminated)*/
   char loc[TRACE2_LOC_LEN];      /* Location code (NULL-terminated) */
   int       Debug;               /* If 1, print debug messages */
   unsigned char MyModId;         /* Module id of this program */
   long      InKey;               /* Key to ring with data messages */
   int       HeartbeatInt;        /* Heartbeat interval in seconds */
   int       OutputInterval;      /* Output Interval in seconds */
   int       NumOutputWindows;    /* Number of Output Windows */
   int       *OutputWindows;      /* Output Windows in seconds */
   char OutputPathname[PATH_MAX]; /* Output path name */
   char OutputCommand[PATH_MAX];  /* Output command */
   int       nGetLogo;            /* Number of logos in GetLogo   */
   MSG_LOGO *GetLogo;             /* Logos of requested messages */
} GPARM;

// Earthworm related parameters
typedef struct {
   pid_t MyPid;
   unsigned char InstId;
   unsigned char TypeError;
   unsigned char TypeHeartBeat;
   unsigned char TypeStrongMotionGM;
   SHM_INFO InRegion;               /* shared memory region to use for input */
   unsigned char InstIdWild;        /* Wildcard for inst id */
   unsigned char ModIdWild;         /* Wildcard for module id */
} EWPARM;

_EXTERN_ GPARM Gparm;
_EXTERN_ EWPARM Ewparm;

// load the configuration files
void config_load(char *configfile);

#endif
