#define __GM_DATA_C__
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <earthworm.h>
#include "config.h"
#include "gm_data.h"

int last_saved_index = -1;
int largest_seconds = 0;
int data_table_size = 0;
PG_TABLE_ENTRY *pg_table = NULL;

int decrement_data_index(int *index)
{
   if (--*index < 0)
   {
      *index = data_table_size - 1;
   }
   return *index;
}

int increment_data_index(int *index)
{
   if (++*index >= data_table_size)
   {
      *index = 0;
   }
   return *index;
}

void get_largest_pg_values(PG_TABLE_ENTRY *largest_pg_values, int time_span)
{
   double pga, pgv;
   int i;
   int index = last_saved_index;
   int numIntervals = time_span / Gparm.OutputInterval;
   largest_pg_values->pga = pg_table[index].pga;
   largest_pg_values->pgv = pg_table[index].pgv;
   for (i = 1; i < numIntervals; i++)
   {
      decrement_data_index(&index);
      pga = pg_table[index].pga;
      pgv = pg_table[index].pgv;
      if (pga > largest_pg_values->pga)
      {
         largest_pg_values->pga = pga;
      }
      if (pgv > largest_pg_values->pgv)
      {
         largest_pg_values->pgv = pgv;
      }
   }
}

int init_gm_data()
{
   memset(&last_strong_motion_gm, 0, sizeof(STRONG_MOTION_GM));
   if (pg_table == NULL)
   {
      // determine the larget window
      int index;
      int seconds;
      for (index = 0; index < Gparm.NumOutputWindows; index++)
      {
         seconds = Gparm.OutputWindows[index];
         if (seconds > largest_seconds)
         {
            largest_seconds = seconds;
         }
      }
      data_table_size = largest_seconds / Gparm.OutputInterval;
      if (Gparm.Debug)
      {
         printf("report_gm: Gparm.OutputInterval=%d\n", Gparm.OutputInterval);
         printf("report_gm: largest_seconds=%d\n", largest_seconds);
         printf("report_gm: data_table_size=%d\n", data_table_size);
      }
      pg_table = calloc(data_table_size, sizeof(PG_TABLE_ENTRY));
      if (pg_table == NULL)
      {
         logit("e", "report_gm: could not allocate pg table <%d>. Exiting.\n", data_table_size);
         return -1;
      }
   }
   else
   {
      if (Gparm.Debug)
      {
         printf("report_gm: data_table_size=%d\n", data_table_size);
      }
   }
   return data_table_size;
}

void save_values(STRONG_MOTION_GM *smg)
{
   int current_index;
   time_t time = smg->time;
   double pga = smg->pga;
   double pgv = smg->pgv;
   if (data_table_size <= 0)
   {
      if (init_gm_data() <= 0)
      {
         return;
      }
   }
   current_index = (time / Gparm.OutputInterval) % data_table_size;
   // if same window as last time
   if (current_index == last_saved_index)
   {
      // udpdate value if larger
      if (pga > pg_table[current_index].pga)
      {
         pg_table[current_index].pga = pga;
         if (Gparm.Debug)
         {
            printf("report_gm: save pga value(%ld, %d, %f)\n", time, current_index, pga);
         }
      }
      if (pgv > pg_table[current_index].pgv)
      {
         pg_table[current_index].pgv = pgv;
         if (Gparm.Debug)
         {
            printf("report_gm: save pgv value(%ld, %d, %f)\n", time, current_index, pgv);
         }
      }
   }
   else
   {
      // if not first value saved
      if (last_saved_index != -1)
      {
         // clear previous table values if gap
         int index = last_saved_index;;
         while ((index = increment_data_index(&index)) != current_index)
         {
            if (Gparm.Debug)
            {
               printf("report_gm: clearing [%d]\n", index);
            }
            memset(pg_table + current_index, 0, sizeof(PG_TABLE_ENTRY));
         }
      }
      pg_table[current_index].pga = pga;
      pg_table[current_index].pgv = pgv;
      memcpy(&last_strong_motion_gm, smg, sizeof(STRONG_MOTION_GM));
      last_saved_index = current_index;
      if (Gparm.Debug)
      {
         printf("report_gm: save_values(%ld, %d, %f, %f)\n", time, current_index, pga, pgv);
      }
   }
}
