#define __WRITE_REPORT_C__
#include <errno.h>
#include <string.h>
#include <earthworm.h>
#include "config.h"
#include "gm_data.h"
#include "write_report.h"

int isTimeToReport(time_t timeNow)
{
   // if no previous report
   if (lastStrongMotionGmReport == 0)
   {
      // start timer
      lastStrongMotionGmReport = timeNow;
      return 0;
   }
   return Gparm.OutputInterval &&
          timeNow - lastStrongMotionGmReport >= Gparm.OutputInterval;
}

void write_report(time_t timeNow)
{
   int i;
   int time_span;
   PG_TABLE_ENTRY largest_pg_values;
   FILE *stream = stdout;

   if (*Gparm.OutputPathname && strcmp("stdout", Gparm.OutputPathname) != 0)
   {
      errno = 0; // clear previous error
      stream = fopen(Gparm.OutputPathname, "w");
      if (stream == NULL)
      {
         if (errno != 0)
         {
            logit("e", "report_gm: Could not open file (%s): %s\n",
                  Gparm.OutputPathname, strerror(errno));
         }
         else
         {
            logit("e", "report_gm: Could not open file (%s)\n",
                  Gparm.OutputPathname);
         }
         return;
      }
   }
   fprintf(stream, "{");
   fprintf(stream, "\"sta\":\"%s\",", Gparm.sta);
   fprintf(stream, "\"chan\":\"%s\",", Gparm.chan);
   fprintf(stream, "\"net\":\"%s\",", Gparm.net);
   fprintf(stream, "\"loc\":\"%s\",", Gparm.loc);
   fprintf(stream, "\"lat\":%g,", last_strong_motion_gm.lat);
   fprintf(stream, "\"lon\":%g,", last_strong_motion_gm.lon);
   fprintf(stream, "\"generated\":%ld,", timeNow);
   if (last_strong_motion_gm.time == (time_t)last_strong_motion_gm.time)
   {
      fprintf(stream, "\"time\":%ld,", (time_t)last_strong_motion_gm.time);
   }
   else
   {
      fprintf(stream, "\"time\":%g,", last_strong_motion_gm.time);
   }
   fprintf(stream, "\"windows\":[");
   for (i = 0; i < Gparm.NumOutputWindows; i++)
   {
      time_span = Gparm.OutputWindows[i];
      get_largest_pg_values(&largest_pg_values, time_span);
      fprintf(stream, "{");
      fprintf(stream, "\"time-span\":%d,", time_span);
      fprintf(stream, "\"pga\":%g,", largest_pg_values.pga);
      fprintf(stream, "\"pgv\":%g", largest_pg_values.pgv);
      if (i != Gparm.NumOutputWindows - 1)
      {
         fprintf(stream, "},");
      }
      else
      {
         fprintf(stream, "}");
      }
   }
   fprintf(stream, "]}\n");
   if (*Gparm.OutputPathname)
   {
      fclose(stream);
   }
   lastStrongMotionGmReport = timeNow;
}
